﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using projectDoubleDeckerBus.Drawings;
using projectDouble_Decker_Bus.MovementStrategy;
using projectDoubleDeckerBus.Generics;
using System.Text;
using projectDoubleDeckerBus.Exceptions;

namespace projectDoubleDeckerBus.Generics
{
    internal class BusesGenericStorage
    {
        readonly Dictionary<string, BusesGenericCollection<DrawningBus, DrawningObjectBus>> _busStorages;
        public List<string> Keys => _busStorages.Keys.ToList();
        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        private static readonly char _separatorForKeyValue = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char _separatorRecords = ';';
        /// <summary>
        /// Разделитель для записи информации по объекту в файл
        /// </summary>
        private static readonly char _separatorForObject = ':';

        public BusesGenericStorage(int pictureWidth, int pictureHeight)
        {
            _busStorages = new Dictionary<string, BusesGenericCollection<DrawningBus, DrawningObjectBus>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        public void AddSet(string name)
        {
            if (_busStorages.ContainsKey(name)) return;
            _busStorages[name] = new BusesGenericCollection<DrawningBus, DrawningObjectBus>(_pictureWidth, _pictureHeight);
        }

        public void DelSet(string name)
        {
            if (!_busStorages.ContainsKey(name)) return;
            _busStorages.Remove(name);
        }

        public BusesGenericCollection<DrawningBus, DrawningObjectBus>?
        this[string ind]
        {
            get
            {
                if (_busStorages.ContainsKey(ind)) return _busStorages[ind];
                return null;
            }
        }
        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string, BusesGenericCollection<DrawningBus, DrawningObjectBus>> record in _busStorages)
            {
                StringBuilder records = new();
                foreach (DrawningBus? elem in record.Value.GetBuses)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");

            }
            if (data.Length == 0)
            {
                throw new Exception("Невалидная операция, нет данных для сохранения");
            }
            using (StreamWriter writer = new StreamWriter(filename))
            {
                writer.WriteLine("BusStorage");
                writer.Write(data.ToString());
            }
        }

        /// <summary>
        /// Загрузка информации по автомобилям в хранилище из файла
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка при

        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new Exception("Файл не найден");
            }
            using (StreamReader reader = new StreamReader(filename))
            {
                string checker = reader.ReadLine();
                if (checker == null)
                    throw new Exception("Нет данных для загрузки");
                if (!checker.StartsWith("BusStorage"))
                    throw new Exception("Неверный формат ввода");
                _busStorages.Clear();
                string strs;
                bool firstinit = true;
                while ((strs = reader.ReadLine()) != null)
                {
                    if (strs == null && firstinit)
                        throw new Exception("Нет данных для загрузки");
                    if (strs == null)
                        break;
                    firstinit = false;
                    string name = strs.Split('|')[0];
                    BusesGenericCollection<DrawningBus, DrawningObjectBus> collection = new(_pictureWidth, _pictureHeight);
                    foreach (string data in strs.Split('|')[1].Split(';'))
                    {
                        DrawningBus? bus =
                        data?.CreateDrawningBus(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (bus != null)
                        {
                            try { _ = collection + bus; }
                            catch (BusNotFoundException e)
                            {
                                throw e;
                            }
                            catch (StorageOverflowException e)
                            {
                                throw e;
                            }
                        }
                    }
                    _busStorages.Add(name, collection);
                }
            }
        }
    }
}





